from .base import BaseContentHandler
import os
import subprocess
import shutil
from urllib.parse import urlparse
from sirji_tools.logger import r_logger as logger

class GitHubHandler(BaseContentHandler):
    def handle(self, url, output_dir):
        logger.info(f"Started handling GitHub URL: {url}")

        parsed_url = urlparse(url)
        repo_name = parsed_url.path.split('/')[-1]
        project_folder = os.path.join(output_dir, "external_resources", repo_name)
        
        # Remove the directory if it already exists
        if os.path.exists(project_folder):
            shutil.rmtree(project_folder)
            logger.info(f"Removed existing directory: {project_folder}")
        
        # Clone the repository
        try:
            subprocess.run(['git', 'clone', url, project_folder], check=True)
            logger.info(f"Repository {repo_name} cloned into {project_folder}")
        except subprocess.CalledProcessError:
            logger.error(f"Failed to clone repository: {url}")
            
        finally:
            logger.info(f"Completed handling GitHub URL: {url}")


from playwright.sync_api import sync_playwright
import time
from markdownify import markdownify as md
import re

from .base import BaseContentHandler
from sirji_tools.logger import r_logger as logger


class WebPageHandler(BaseContentHandler):
    def handle(self, url, output_dir):
        with sync_playwright() as pw:
            delay_seconds = 10

            logger.info(f"Started crawling web page URL: {url}")
            browser = pw.chromium.launch()

            context = browser.new_context(
                user_agent="Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3"
            )
            page = context.new_page()

            page.goto(url, timeout=120000)

            time.sleep(delay_seconds)


            # Extract the content from the webpage body only
            body_content = page.query_selector('body').inner_html()

            # Remove script tags
            body_content = re.sub(r'<script.*?</script>',
                                  '', body_content, flags=re.DOTALL)

            # Convert the content to markdown format
            markdown_content = md(body_content)

            self.save_content(markdown_content, url, output_dir, 'md')

            browser.close()

            logger.info(f"Completed crawling web page URL: {url}")



from .logger import LoggerManager

c_logger = LoggerManager().coder
r_logger = LoggerManager().researcher
p_logger = LoggerManager().planner
e_logger = LoggerManager().executor
s_logger = LoggerManager().sirji
u_logger = LoggerManager().user



import pytest
from unittest.mock import patch, MagicMock
from sirji_tools.logger import LoggerManager

class TestLoggerManager:
    @patch('sirji_tools.logger.logger.logging')
    def test_logger_manager_initialization(self, mock_logging):
        # Setup
        mock_logging.getLogger.return_value = MagicMock()

        # Test LoggerManager initialization
        logger_manager = LoggerManager()

        # Assertions to ensure that logger instances are created
        assert logger_manager.coder is not None, "LoggerManager did not initialize a coder logger."
        assert logger_manager.researcher is not None, "LoggerManager did not initialize a researcher logger."
        assert logger_manager.planner is not None, "LoggerManager did not initialize a planner logger."
        assert logger_manager.executor is not None, "LoggerManager did not initialize an executor logger."
        assert logger_manager.sirji is not None, "LoggerManager did not initialize a sirji logger."
        assert logger_manager.user is not None, "LoggerManager did not initialize a user logger."

    @patch('sirji_tools.logger.logger.logging')
    def test_logger_manager_logging(self, mock_logging):
        # Setup
        mock_logger = MagicMock()
        mock_logging.getLogger.return_value = mock_logger

        # Test logging through LoggerManager
        logger_manager = LoggerManager()
        logger_manager.coder.info("Test message for coder")
        logger_manager.researcher.info("Test message for researcher")
        logger_manager.planner.info("Test message for planner")
        logger_manager.executor.info("Test message for executor")
        logger_manager.sirji.info("Test message for sirji")
        logger_manager.user.info("Test message for user")

        # Verify
        mock_logger.info.assert_any_call("Test message for coder"), "LoggerManager's coder did not log message correctly."
        mock_logger.info.assert_any_call("Test message for researcher"), "LoggerManager's researcher did not log message correctly."
        mock_logger.info.assert_any_call("Test message for planner"), "LoggerManager's planner did not log message correctly."
        mock_logger.info.assert_any_call("Test message for executor"), "LoggerManager's executor did not log message correctly."
        mock_logger.info.assert_any_call("Test message for sirji"), "LoggerManager's sirji did not log message correctly."
        mock_logger.info.assert_any_call("Test message for user"), "LoggerManager's user did not log message correctly."


import os
import logging
import time

# Mapping of log level strings to constants
_log_level_str_to_const = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'critical': logging.CRITICAL
}

# Set default log level to DEBUG
_default_log_level = _log_level_str_to_const.get(os.environ.get("SIRJI_LOG_LEVEL", 'debug').lower())

class UnixTimestampFormatter(logging.Formatter):
    def formatTime(self, record, datefmt=None):
        return f"{int(time.time())}"
    
# Singleton class to create loggers
class LoggerSingleton:

    def __init__(self, file_name, log_level):
        self.logger = self._setup_logger(file_name, log_level)
    
    def _log_folder(self):
        return os.path.join(self._get_workspace_folder(), '.sirji', self._get_run_id_folder(), "logs")
    
    def _get_workspace_folder(self):
        workspace = os.environ.get("SIRJI_WORKSPACE")
        if workspace is None:
            raise ValueError(
                "SIRJI_WORKSPACE is not set as an environment variable")
        return workspace

    def _get_run_id_folder(self):
        run_id = os.environ.get("SIRJI_RUN_ID")
        if run_id is None:
            raise ValueError(
                "SIRJI_RUN_ID is not set as an environment variable")
        return run_id 
    
    def _log_file_path(self, file_name):
        return os.path.join(self._log_folder(), file_name)
        
    def _setup_logger(self, file_name, log_level):
        # Create a folder named "logs" if it doesn't exist
        if not os.path.exists(self._log_folder()):
            os.makedirs(self._log_folder())
        
        logger = logging.getLogger(file_name)
        logger.setLevel(log_level)
        if not logger.handlers:
            # Ensure no duplicate handlers
            file_handler = logging.FileHandler(self._log_file_path(file_name))
            formatter = UnixTimestampFormatter('[%(asctime)s] %(message)s')
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)
        
        logger.filepath = self._log_file_path(file_name)

        # Attach the initializeLogs method
        def initialize_logs(self, msg):
            original_formatter = self.handlers[0].formatter
            simple_formatter = logging.Formatter('%(message)s')  # Formatter without Unix timestamp
            try:
                # Temporarily change formatter to log without timestamp
                self.handlers[0].setFormatter(simple_formatter)
                self.info(msg)
            finally:
                # Revert to original formatter
                self.handlers[0].setFormatter(original_formatter)
        
        # Monkey patch the logger instance
        logger.initialize_logs = initialize_logs.__get__(logger)
        
        return logger

# Manager class to create and manage loggers
class LoggerManager:
    def __init__(self):
        self._coder = LoggerSingleton('coder.log', _default_log_level).logger
        self._researcher = LoggerSingleton('researcher.log', _default_log_level).logger
        self._planner = LoggerSingleton('planner.log', _default_log_level).logger
        self._executor = LoggerSingleton('executor.log', _default_log_level).logger
        self._sirji = LoggerSingleton('sirji.log', _default_log_level).logger
        self._user = LoggerSingleton('user.log', _default_log_level).logger

    @property
    def coder(self):
        return self._coder

    @property
    def researcher(self):
        return self._researcher
    
    @property
    def planner(self):
        return self._planner
    
    @property
    def executor(self):
        return self._executor
    
    @property
    def sirji(self):
        return self._sirji
    
    @property
    def user(self):
        return self._user

logger = LoggerManager()